#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Утилита настройки подключения к ftp-серверу
"""
import subprocess
import dialog
import os

auto_ftp_filename = "/etc/auto.ftp"
auto_master_filename = "/etc/auto.master"
netrc_filename = "/root/.netrc"
mount_directory_path = "/linuxcash/net/server"

class Connection:
	"""
	Класс настроек для ftp-соединения
	"""

	def __init__(self, address="", login="", password="", timeout=""):
		self.init(address, login, password, timeout)

	def init(self, address="", login="", password="", timeout=""):
		self.address = address
		self.login = login
		self.password = password
		self.timeout = timeout

	def copy(self):
		"""
		Вернуть копию объекта Connection
		"""
		connection = Connection(
			self.address,
			self.login,
			self.password,
			self.timeout
			)
		return connection

	@staticmethod
	def is_valid(connection):
		"""
		Проверить, является ли правильно заполненным объект класса Connection
		"""
		if not connection.address:
			return False
		if not connection.login:
			return False
		if not connection.password:
			return False
		if not connection.timeout or not connection.timeout.isdigit():
			return False
		return True

	@staticmethod
	def make_valid_address(address):
		"""
		Получить валидный адрес
		"""
		valid_address = address.replace("ftp://", "")
		return valid_address


class FtpPropertiesReader:
	"""
	Класс для чтения конфигурационных файлов для настройки ftp
	"""

	def __init__(self):
		"""
		Конструктор объекта класса FtpPropertiesReader
		"""
		self._connection = Connection()

	def set_connection(self, connection):
		"""
		Задать объект класса Connection
		"""
		self._connection = connection

	def get_connection(self):
		"""
		Получить объект класса Connection
		"""
		return self._connection

	def load_properties(self, auto_master_filename, netrc_filename):
		"""
		Загрузить настройки из конфигурационных файлов
		"""
		self._load_from_netrc(netrc_filename)
		self._load_from_auto_master(auto_master_filename)

	def _cut_double_spaces_and_tabs(self, line):
		"""
		Вырезать из строки символы табуляции и двойных пробелов, заменив их на одинарные
		"""
		line = line.strip()
		while "  " in line or "\t" in line:
			line = line.replace("  ", " ").replace("\t", " ")
		return line

	def _load_from_netrc(self, netrc_filename):
		"""
		Загрузить настройки из файла .netrc
		"""
		if os.path.exists(netrc_filename):
			netrc_fd = open(netrc_filename)
			content = netrc_fd.readlines()
			netrc_fd.close()
			for line in content:
				line = self._cut_double_spaces_and_tabs(line)
				words = line.split(' ')
				if words[0] == "machine":
					self._connection.address = words[1]
				if words[0] == "login":
					self._connection.login = words[1]
				if words[0] == "password":
					self._connection.password = words[1]
			return True
		else:
			return False

	def _load_from_auto_master(self, auto_master_filename):
		"""
		Загрузить настройки из файла auto.master
		"""
		if os.path.exists(auto_master_filename):
			master_fd = open(auto_master_filename)
			content = master_fd.readlines()
			master_fd.close()
			for line in content:
				line = line.strip()
				if line and line[0] is not '#' and "auto.ftp" in line:
					line = self._cut_double_spaces_and_tabs(line)
					for word in line.split(' '):
						if "--timeout=" in word:
							values = word.split('=')
							self._connection.timeout = values[1]
			return True
		else:
			return False


class FtpPropertiesWriter:
	"""
	Класс для записи конфигурационных файлов для настройки ftp
	"""
	def __init__(self):
		"""
		Конструктор объекта класса FtpPropertiesReader
		"""
		self._connection = Connection()

	def set_connection(self, connection):
		"""
		Задать объект класса Connection
		"""
		self._connection = connection

	def get_connection(self):
		"""
		Получить объект класса Connection
		"""
		return self._connection

	def _save_netrc(self, netrc_filename):
		"""
		Сохранить настройки в файл .netrc
		"""
		netrc_fd = open(netrc_filename, "w")
		netrc_content = "machine " + self._connection.address + "\n"
		netrc_content += "login " + self._connection.login + "\n"
		netrc_content += "password " + self._connection.password
		netrc_fd.write(netrc_content)
		netrc_fd.close()
		return True

	def _save_auto_ftp(self, auto_ftp_filename):
		"""
		Сохранить настройки в файл auto.ftp
		"""
		auto_fd = open(auto_ftp_filename, "w")
		address = self.get_connection().address.replace("ftp://", "ftp\\://")
		auto_fd.write("ftp -fstype=curl :" + address)
		auto_fd.close()
		return True

	def _save_auto_master(self, auto_master_filename, auto_ftp_filename, mount_directory_path):
		"""
		Сохранить настройки в файл auto.master
		"""
		content = mount_directory_path + " " + auto_ftp_filename + " --timeout=" + self.get_connection().timeout + " --ghost"
		if os.path.exists(auto_master_filename):
			fd_master = open(auto_master_filename)
			fd_content = fd_master.readlines()
			fd_master.close()
			is_writed = False
			output_content = ""
			for line in fd_content:
				line2 = line.strip()
				if line2 and line2[0] is not '#' and mount_directory_path in line2:
					output_content += content + "\n"
					is_writed = True
				else:
					output_content += line
			if not is_writed:
				output_content += content
		else:
			output_content = content
		fd_master = open(auto_master_filename, "w")
		fd_master.write(output_content)
		fd_master.close()
		return True

	def save_properties(self, auto_ftp_filename, auto_master_filename, netrc_filename, mount_directory_path):
		"""
		Сохранить настройки в конфигурационных файлах
		"""
		self._save_netrc(netrc_filename)
		self._save_auto_ftp(auto_ftp_filename)
		self._save_auto_master(auto_master_filename, auto_ftp_filename, mount_directory_path)


def main_form(d, connection):
	"""
	Главная форма настройки ftp-подключения
	"""
	is_valid = False
	is_cancel = False
	current_connection = connection.copy()
	while not is_valid and not is_cancel:
		code, tag = d.form(
			text="Введите параметры ftp-подключения",
			height=11,
			form_height=4,
			fields=[
				("Адрес:", current_connection.address, 16, 60),
				("Логин:", current_connection.login, 16, 60),
				("Пароль:", current_connection.password, 16, 60),
				("Таймаут:", current_connection.timeout, 16, 60)
			]
		)
		
		if code == d.DIALOG_OK:
			current_connection.init(tag[0], tag[1], tag[2], tag[3])

			current_connection.address = Connection.make_valid_address(current_connection.address)

			is_valid = Connection.is_valid(current_connection)

			if not is_valid:
				d.msgbox("Неправильно заполнены параметры", height=7)
		else:
			is_cancel = True
	return is_cancel, current_connection


def main():
	"""
	Точка входа
	"""
	d = dialog.Dialog()
	d.add_persistent_args(("--backtitle", "Настройка ftp-подключения"))

	ftp_propeties_reader = FtpPropertiesReader()
	ftp_propeties_reader.load_properties(
		auto_master_filename=auto_master_filename,
		netrc_filename=netrc_filename
	)

	is_cancel, connection = main_form(d, ftp_propeties_reader.get_connection())

	if not is_cancel:
		ftp_propeties_writer = FtpPropertiesWriter()
		ftp_propeties_writer.set_connection(connection)
		ftp_propeties_writer.save_properties(
			auto_ftp_filename,
			auto_master_filename,
			netrc_filename,
			mount_directory_path
		)
		subprocess.Popen(["/etc/init.d/autofs", "restart"], stdout = subprocess.PIPE, stderr=subprocess.PIPE)

if __name__ == "__main__":
	os.chdir(os.path.dirname(os.path.abspath(__file__)))
	main()